@@ -7,7 +7,8 @@ __all__ = [
     'ConnectionTerminatedError', 'WirError', 'WebInspectorNotEnabled', 'RemoteAutomationNotEnabled',
     'ArbitrationError', 'NoSuchBuildIdentityError', 'InternalError', 'DeveloperModeIsNotEnabledError',
     'DeviceAlreadyInUseError', 'LockdownError', 'PairingDialogResponsePendingError', 'UserDeniedPairingError',
-    'InvalidHostIDError', 'SetProhibitedError', 'MissingValueError',
+    'InvalidHostIDError', 'SetProhibitedError', 'MissingValueError', 'PasscodeRequiredError', 'AmfiError',
+    'DeviceHasPasscodeSetError',
 ]
 
 
@@ -201,3 +202,16 @@ class InvalidHostIDError(PairingError):
 class MissingValueError(LockdownError):
     """ raised when attempting to query non-existent domain/key """
     pass
+
+
+class PasscodeRequiredError(LockdownError):
+    """ passcode must be present for this action """
+    pass
+
+
+class AmfiError(PyMobileDevice3Exception):
+    pass
+
+
+class DeviceHasPasscodeSetError(AmfiError):
+    pass
