@@ -5,18 +5,18 @@ import os
 import platform
 import plistlib
 import sys
+import tempfile
 import time
 import uuid
-from contextlib import suppress
+from contextlib import suppress, contextmanager
 from pathlib import Path
-from typing import Mapping
+from typing import Mapping, Union, Optional
 
 from packaging.version import Version
-
 from pymobiledevice3 import usbmux
 from pymobiledevice3.ca import ca_do_everything
 from pymobiledevice3.exceptions import *
-from pymobiledevice3.exceptions import LockdownError, SetProhibitedError
+from pymobiledevice3.exceptions import LockdownError, SetProhibitedError, PasscodeRequiredError
 from pymobiledevice3.service_connection import ServiceConnection
 from pymobiledevice3.usbmux import PlistMuxConnection
 from pymobiledevice3.utils import sanitize_ios_version
@@ -33,13 +33,6 @@ LOCKDOWN_PATH = {
 }
 
 
-def write_home_file(filename, data):
-    HOMEFOLDER.mkdir(parents=True, exist_ok=True)
-    filepath = HOMEFOLDER / filename
-    filepath.write_bytes(data)
-    return str(filepath)
-
-
 def reconnect_on_remote_close(f):
     """
     lockdownd's _socket_select will close the connection after 60 seconds of "radio-silent" (no data has been
@@ -106,7 +99,7 @@ class LockdownClient(object):
 
     def __init__(self, udid: str = None, client_name: str = DEFAULT_CLIENT_NAME, autopair: bool = True,
                  connection_type: str = None, pair_timeout: int = None, hostname: str = None,
-                 pair_record: Mapping = None):
+                 pair_record: Mapping = None, pairing_records_cache_folder: Path = HOMEFOLDER):
         """
         :param udid: serial number for device to connect to
         :param client_name: user agent to use when identifying for lockdownd
@@ -132,7 +125,7 @@ class LockdownClient(object):
         self.system_buid = SYSTEM_BUID
         self.label = client_name
         self.pair_record = pair_record
-        self.ssl_file = None
+        self.pairing_records_cache_folder = pairing_records_cache_folder
 
         if self.query_type() != 'com.apple.mobile.lockdown':
             raise IncorrectModeError()
@@ -220,6 +213,17 @@ class LockdownClient(object):
     def invert_display(self, value: bool):
         self.set_value(int(value), 'com.apple.Accessibility', 'InvertDisplayEnabledByiTunes')
 
+    @property
+    def enable_wifi_pairing(self) -> bool:
+        return self.get_value('com.apple.mobile.wireless_lockdown').get('EnableWifiPairing', False)
+
+    @enable_wifi_pairing.setter
+    def enable_wifi_pairing(self, value: bool):
+        try:
+            self.set_value(value, 'com.apple.mobile.wireless_lockdown', 'EnableWifiPairing')
+        except MissingValueError as e:
+            raise PasscodeRequiredError from e
+
     @property
     def enable_wifi_connections(self):
         return self.get_value('com.apple.mobile.wireless_lockdown').get('EnableWifiConnections', False)
@@ -276,7 +280,7 @@ class LockdownClient(object):
                 raise CannotStopSessionError()
             return response
 
-    def get_itunes_pairing_record(self):
+    def get_itunes_pairing_record(self) -> Optional[Mapping]:
         platform_type = 'linux' if not sys.platform.startswith('linux') else sys.platform
         filename = LOCKDOWN_PATH[platform_type] / f'{self.identifier}.plist'
         try:
@@ -288,7 +292,7 @@ class LockdownClient(object):
 
     def get_local_pairing_record(self):
         self.logger.debug('Looking for pymobiledevice3 pairing record')
-        path = HOMEFOLDER / f'{self.identifier}.plist'
+        path = self.pairing_records_cache_folder / f'{self.identifier}.plist'
         if not path.exists():
             self.logger.error(f'No pymobiledevice3 pairing record found for device {self.identifier}')
             return None
@@ -303,9 +307,6 @@ class LockdownClient(object):
         if self.pair_record is None:
             return False
 
-        cert_pem = self.pair_record['HostCertificate']
-        private_key_pem = self.pair_record['HostPrivateKey']
-
         if Version(self.product_version) < Version('11.0'):
             try:
                 self._request('ValidatePair', {'PairRecord': self.pair_record})
@@ -323,9 +324,8 @@ class LockdownClient(object):
 
         self.session_id = start_session.get('SessionID')
         if start_session.get('EnableSessionSSL'):
-            lf = b'\n'
-            self.ssl_file = write_home_file(f'{self.identifier}_ssl.txt', cert_pem + lf + private_key_pem)
-            self.service.ssl_start(self.ssl_file, self.ssl_file)
+            with self._ssl_file() as f:
+                self.service.ssl_start(f)
 
         self.paired = True
         return True
@@ -356,7 +356,7 @@ class LockdownClient(object):
         pair_record['HostPrivateKey'] = private_key_pem
         pair_record['EscrowBag'] = pair.get('EscrowBag')
         self.pair_record = pair_record
-        write_home_file(f'{self.identifier}.plist', plistlib.dumps(pair_record))
+        self._write_storage_file(f'{self.identifier}.plist', plistlib.dumps(pair_record))
 
         record_data = plistlib.dumps(pair_record)
 
@@ -371,6 +371,10 @@ class LockdownClient(object):
     def unpair(self) -> Mapping:
         return self._request('Unpair', {'PairRecord': self.pair_record, 'ProtocolVersion': '2'}, verify_request=False)
 
+    @reconnect_on_remote_close
+    def reset_pairing(self):
+        return self._request('ResetPairing', {'FullReset': True})
+
     @reconnect_on_remote_close
     def get_value(self, domain: str = None, key: str = None):
         options = {}
@@ -432,15 +436,18 @@ class LockdownClient(object):
         service_connection = ServiceConnection.create(self.udid, attr['Port'],
                                                       connection_type=self.usbmux_device.connection_type)
         if attr.get('EnableServiceSSL', False):
-            service_connection.ssl_start(self.ssl_file, self.ssl_file)
+            with self._ssl_file() as f:
+                service_connection.ssl_start(f)
         return service_connection
 
     async def aio_start_service(self, name, escrow_bag=None) -> ServiceConnection:
         attr = self.get_service_connection_attributes(name, escrow_bag=escrow_bag)
         service_connection = ServiceConnection.create(self.udid, attr['Port'],
                                                       connection_type=self.usbmux_device.connection_type)
+
         if attr.get('EnableServiceSSL', False):
-            await service_connection.aio_ssl_start(self.ssl_file, self.ssl_file)
+            with self._ssl_file() as f:
+                service_connection.aio_ssl_start(f)
         return service_connection
 
     def start_developer_service(self, name, escrow_bag=None) -> ServiceConnection:
@@ -456,6 +463,11 @@ class LockdownClient(object):
     def close(self):
         self.service.close()
 
+    def _write_storage_file(self, filename: Union[Path, str], data: bytes) -> None:
+        self.pairing_records_cache_folder.mkdir(parents=True, exist_ok=True)
+        filepath = self.pairing_records_cache_folder / filename
+        filepath.write_bytes(data)
+
     def _request(self, request: str, options: Mapping = None, verify_request: bool = True) -> Mapping:
         message = {'Label': self.label, 'Request': request}
         if options:
@@ -516,3 +528,13 @@ class LockdownClient(object):
             return
 
         self.pair_record = pair_record
+
+    @contextmanager
+    def _ssl_file(self) -> str:
+        cert_pem = self.pair_record['HostCertificate']
+        private_key_pem = self.pair_record['HostPrivateKey']
+
+        with tempfile.NamedTemporaryFile('w+b') as f:
+            f.write(cert_pem + b'\n' + private_key_pem)
+            f.flush()
+            yield f.name
